#!/usr/bin/python
#
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This example creates a query for the specified customer.

For additional sample queries you can use to get started, see:
https://developers.google.com/ads-data-hub/samples/writing-queries
"""

import argparse
import pprint
import random
import sys

from googleapiclient.errors import HttpError
import samples_util


DEFAULT_CUSTOMER_ID = 'INSERT_DEFAULT_CUSTOMER_ID_HERE'
DEFAULT_QUERY_TEXT = """WITH stats AS (
  SELECT
    campaign_id,
    demographics.gender AS gender_id,
    demographics.age_group AS age_group_id,
    COUNT(DISTINCT user_id) AS users,
    COUNT(*) AS impressions
  FROM adh.adwords_impressions
  WHERE customer_id = @customer_id
  GROUP BY 1, 2, 3
)
SELECT
  campaign_name,
  gender_name,
  age_group_name,
  users,
  impressions
FROM stats
  LEFT JOIN adh.adwords_campaign USING (campaign_id)
  LEFT JOIN adh.gender USING (gender_id)
  LEFT JOIN adh.age_group USING (age_group_id)
ORDER BY 1, 2, 3
"""
DEFAULT_PARAMETER_TYPES = {
    'customer_id': {
        'type': 'INT64'
    }
}


def main(full_circle_query, customer_id, body):
  try:
    # Execute the request.
    new_query = full_circle_query.queries().create(
        body=body).execute()
  except HttpError as e:
    print e
    sys.exit(1)

  print 'New query created for customer ID "%s":' % customer_id
  pprint.pprint(new_query)


if __name__ == '__main__':
  parser = argparse.ArgumentParser(
      description='Creates a new query for given Ads Data Hub customer.')
  parser.add_argument(
      '-c', '--customer_id', default=DEFAULT_CUSTOMER_ID,
      help=('The ID of the Ads Data Hub customer for which you are creating a '
            'new query.'))
  parser.add_argument(
      '-n', '--query_name',
      default='AW_Demographics_Example_%s' % random.randint(10000000, 99999999),
      help=('The unique name to be associated with the new query.'))
  parser.add_argument('-v', '--validate_only', default=False, type=bool,
                      help=('If set True, the submitted query will only be '
                            'validated. A new query will not be created.'))
  args = parser.parse_args()

  try:
    service = samples_util.GetService()
  except IOError, ex:
    print 'Unable to create ads data hub service - %s' % ex
    print 'Did you specify the client secrets file in samples_util.py?'
    sys.exit(1)

  query_create_body = {
      'customerId': args.customer_id,
      'validateOnly': args.validate_only,
      'query': {
          'queryName': args.query_name,
          'queryText': DEFAULT_QUERY_TEXT,
          'parameterTypes': DEFAULT_PARAMETER_TYPES
      }
  }

  main(service, args.customer_id, query_create_body)
