#!/usr/bin/python
#
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities used to step through OAuth 2.0 flow.

These are intended to be used for stepping through samples for the Full Circle
Query v2 API.
"""

import json
import sys

from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient import discovery
from oauthlib.oauth2.rfc6749.errors import InvalidGrantError
from google.auth.transport.requests import AuthorizedSession
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials


_APPLICATION_NAME = 'FullCircleQuery v2 Samples'
_CREDENTIALS_FILE = 'fcq-credentials.json'
_SCOPES = 'https://www.googleapis.com/auth/adsdatahub'
_DISCOVERY_URL_TEMPLATE = 'https://%s/$discovery/rest?version=%s&key=%s'
_FCQ_DISCOVERY_FILE = 'fcq-discovery.json'
_FCQ_SERVICE = 'fullcirclequery.googleapis.com'
_FCQ_VERSION = 'v2'
_REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'
_SCOPE = ['https://www.googleapis.com/auth/adsdatahub']
_TOKEN_URI = 'https://accounts.google.com/o/oauth2/token'

# The client secrets file can be downloaded from the Google Cloud Console. For
# more details on configuring your project for access to Full Circle Query v2,
# see the README.
CLIENT_SECRETS_FILE = 'INSERT_PATH_TO_CLIENT_SECRETS_HERE'
# The Developer Key is used to retrieve a discovery document containing the
# non-public Full Circle Query v2 API. This is used to build the service used
# in the samples to make API requests. Please see the README for instructions
# on how to configure your Google Cloud Project for access to the Full Circle
# Query v2 API.
DEVELOPER_KEY = 'INSERT_DEVELOPER_KEY_HERE'
MAX_PAGE_SIZE = 50


def _GetCredentialsFromInstalledApplicationFlow():
  """Get new credentials using the installed application flow."""
  flow = InstalledAppFlow.from_client_secrets_file(
      CLIENT_SECRETS_FILE, scopes=_SCOPE)
  flow.redirect_uri = _REDIRECT_URI  # Set the redirect URI used for the flow.

  auth_url, _ = flow.authorization_url(prompt='consent')

  print ('Log into the Google Account you use to access the Full Circle Query '
         'v2 API and go to the following URL:\n%s\n' % auth_url)
  print 'After approving the token, enter the verification code (if specified).'
  code = raw_input('Code: ')

  try:
    flow.fetch_token(code=code)
  except InvalidGrantError as ex:
    print 'Authentication has failed: %s' % ex
    sys.exit(1)

  credentials = flow.credentials
  _SaveCredentials(credentials)
  return credentials


def _LoadCredentials():
  """Loads and instantiates Credentials from JSON credentials file."""
  with open(_CREDENTIALS_FILE, 'rb') as handler:
    stored_creds = json.loads(handler.read())

  creds = Credentials(client_id=stored_creds['client_id'],
                      client_secret=stored_creds['client_secret'],
                      token=None,
                      refresh_token=stored_creds['refresh_token'],
                      token_uri=_TOKEN_URI)

  return creds


def _SaveCredentials(creds):
  """Save credentials to JSON file."""
  stored_creds = {
      'client_id': getattr(creds, '_client_id'),
      'client_secret': getattr(creds, '_client_secret'),
      'refresh_token': getattr(creds, '_refresh_token')
  }

  with open(_CREDENTIALS_FILE, 'wb') as handler:
    handler.write(json.dumps(stored_creds))


def GetCredentials():
  """Get stored credentials if they exist, otherwise return new credentials.

  If no stored credentials are found, new credentials will be produced by
  stepping through the Installed Application OAuth 2.0 flow with the specified
  client secrets file. The credentials will then be saved for future use.

  Returns:
    A configured google.oauth2.credentials.Credentials instance.
  """
  try:
    creds = _LoadCredentials()
    creds.refresh(Request())
  except IOError:
    creds = _GetCredentialsFromInstalledApplicationFlow()

  return creds


def GetDiscoveryDocument():
  """Downloads the Full Circle Query v2 discovery document.

  Downloads the Full Circle Query v2 discovery document to fcq-discovery.json
  if it is accessible. If the file already exists, it will be overwritten.

  Raises:
    ValueError: raised if the discovery document is inaccessible for any reason.
  """
  credentials = GetCredentials()
  discovery_url = _DISCOVERY_URL_TEMPLATE % (
      _FCQ_SERVICE, _FCQ_VERSION, DEVELOPER_KEY)

  auth_session = AuthorizedSession(credentials)

  discovery_response = auth_session.get(discovery_url)

  if discovery_response.status_code == 200:
    with open(_FCQ_DISCOVERY_FILE, 'wb') as handler:
      handler.write(discovery_response.text)
  else:
    raise ValueError('Unable to retrieve discovery document for api name "%s"'
                     'and version "%s" via discovery URL: %s'
                     % _FCQ_SERVICE, _FCQ_VERSION, discovery_url)


def GetService():
  """Builds a configured Full Circle Query v2 API service.

  Returns:
    A googleapiclient.discovery.Resource instance configured for the Full Circle
      Query v2 service.
  """
  credentials = GetCredentials()
  discovery_url = _DISCOVERY_URL_TEMPLATE % (
      _FCQ_SERVICE, _FCQ_VERSION, DEVELOPER_KEY)

  service = discovery.build(
      'fullcirclequery', _FCQ_VERSION, credentials=credentials,
      discoveryServiceUrl=discovery_url)
  return service


def GetServiceFromDiscoveryDocument():
  """Builds a configured Full Circle Query v2 API service via discovery file.

  Returns:
    A googleapiclient.discovery.Resource instance configured for the Full Circle
      Query API v2 service.
  """
  credentials = GetCredentials()

  with open(_FCQ_DISCOVERY_FILE, 'rb') as handler:
    discovery_doc = handler.read()

  service = discovery.build_from_document(
      service=discovery_doc, credentials=credentials)

  return service
